#include "EGLInterface.h"
#include <assert.h>


void EGLInterface::initEGL() {
	
}

void EGLInterface::shutdownEGL(EGL_STATE_T *descriptor)  {
	
}

void EGLInterface::createEGLContext(EGL_STATE_T *descriptor) {
	EGLBoolean result;
    EGLint num_config;
    EGLint major, minor;
    int maxWidth, maxHeight;


    static const EGLint attribute_list[] =
    {
      EGL_RED_SIZE, 8,
      EGL_GREEN_SIZE, 8,
      EGL_BLUE_SIZE, 8,
      EGL_ALPHA_SIZE, 8,
      EGL_DEPTH_SIZE, 24,
      EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
      EGL_SURFACE_TYPE, EGL_PIXMAP_BIT,
      EGL_NONE
    };
   EGLint context_attribs [] = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE }; //  fuer OpenGL ES 1.1
   
	// get an EGL display connection
	descriptor->display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
	assert(descriptor->display != EGL_NO_DISPLAY);

	// initialize the EGL display connection
	result = eglInitialize(descriptor->display, &major, &minor);
	assert(EGL_FALSE != result);

	// get an appropriate EGL frame buffer configuration
	result = eglChooseConfig(descriptor->display,
	                         attribute_list,
	                         &descriptor->config,
	                         1, 
	                         &num_config);
	assert(EGL_FALSE != result);

	eglBindAPI(EGL_OPENGL_ES_API);

    result = eglGetConfigAttrib(descriptor->display, &descriptor->config, EGL_MAX_PBUFFER_WIDTH, &maxWidth);
    result = eglGetConfigAttrib(descriptor->display, &descriptor->config, EGL_MAX_PBUFFER_HEIGHT, &maxHeight);


	// create an EGL rendering context
	descriptor->context = eglCreateContext(descriptor->display,
	                                       descriptor-> config,
	                                       EGL_NO_CONTEXT,
	                                       context_attribs);
	assert(descriptor->context != EGL_NO_CONTEXT);
	descriptor->hasDisplay = EGL_TRUE;
}


void EGLInterface::freeEGLContext(EGL_STATE_T *descriptor) {
    eglMakeCurrent(descriptor->display,
                   EGL_NO_SURFACE,
                   EGL_NO_SURFACE,
                   EGL_NO_CONTEXT);
    eglDestroySurface(descriptor->display, descriptor->surface);
    eglDestroyContext(descriptor->display, descriptor->context);
    eglTerminate(descriptor->display);
    descriptor-> hasDisplay = EGL_FALSE;
	eglReleaseThread();
}


void EGLInterface::setUpEGLBuffer (EGL_STATE_T *descriptor) {
    EGLint bufferAttribList [] = {
       EGL_WIDTH,  (EGLint)descriptor -> image_width,
       EGL_HEIGHT, (EGLint)descriptor -> image_width,
       EGL_LARGEST_PBUFFER, EGL_TRUE,
       EGL_NONE
    };
    EGLBoolean result; 
    descriptor->surface = eglCreatePbufferSurface(descriptor->display,
                                                  descriptor->config,
                                                  bufferAttribList);
    assert(descriptor->surface != EGL_NO_SURFACE);

    // connect the context to the surface
    result = eglMakeCurrent(descriptor->display,
                            descriptor->surface,
                            descriptor->surface,
                            descriptor->context);
    assert(EGL_FALSE != result);
}

void EGLInterface::resizeEGLBuffer(EGL_STATE_T *descriptor) {
	if (descriptor -> surface != (EGLSurface)0) {
        releaseEGLBuffer(descriptor);
	}
	setUpEGLBuffer(descriptor);
}

void EGLInterface::releaseEGLBuffer(EGL_STATE_T *descriptor) {
    eglMakeCurrent(descriptor->display,
                   EGL_NO_SURFACE,
                   EGL_NO_SURFACE,
                   EGL_NO_CONTEXT);
    eglDestroySurface(descriptor->display,
                      descriptor->surface);
}
