#include "GPUCodeManager.h"
#include "GLES20ErrorHandler.h"
#include <iostream>
#include <cstdio>  // für printf

const char* GPUCodeManager::className = "GPUCodeManager";

GPUCodeManager::GPUCodeManager(ApplicationModel &m) :
   model(m)
{
}

bool GPUCodeManager::loadAndCompileShader(const char* fileName, GLuint& shaderId, GLenum shaderType) {
	bool res = true;
	FILE* f = fopen(fileName, "rb");
	if (f == NULL) {
		return false;
	}
    fseek(f,0,SEEK_END);
    int fSize = ftell(f);
    fseek(f,0,SEEK_SET);
    {
        GLchar codeString[fSize + 1];
        fread(codeString, 1, fSize, f);
        codeString[fSize] = 0; //null terminate it!
        fclose(f);
        GLchar* refString = &codeString[0];
        GLint compileStatus;
	    std::cout << codeString << std::endl;
	    std::cout << " ---------------------------------------------" << std::endl;
	    shaderId  = glCreateShader(shaderType);
	    std::cout << "shader created:" << shaderId << std::endl;
        GLES20ErrorHandler::checkError(GPUCodeManager::className, "glCreateShader", "GL_VERTEX_SHADER");
        std::cout << " load shader source " << std::endl;
        glShaderSource(shaderId, 1, (const GLchar**)&refString, 0);
        GLES20ErrorHandler::checkError(className, "glCreateShader", "GL_VERTEX_SHADER");
        std::cout << " compile shader " << std::endl;
        glCompileShader(shaderId);
        GLES20ErrorHandler::checkError(className, "glCreateShader", "GL_VERTEX_SHADER");


        std::cout << "access compile status" << std::endl;
        glGetShaderiv(shaderId, GL_COMPILE_STATUS, &compileStatus);
        std::cout << "compile status accessed" << compileStatus << std::endl;
        GLES20ErrorHandler::checkError(className, "glCreateShader", "GL_VERTEX_SHADER");
        if (!compileStatus) {
	    	res = false;
		    std::cout << "compile status: " << compileStatus << std::endl;
		    GLint infoLength = 0;
		    glGetShaderiv(shaderId, GL_INFO_LOG_LENGTH, &infoLength);
		    if (infoLength > 1) {
			    GLchar infoBuffer[infoLength + 10];
			    glGetShaderInfoLog(shaderId, infoLength, NULL, infoBuffer);
                printf("shader:\n%s\n", infoBuffer);
		    }
		    glDeleteShader(shaderId);
		    std::cout << "compilation failure in shader" << std::endl;
		}
	}
	std::cout << "done " << std::endl;
	return res;
}
/*
 * Compilierung des Vertex Shaders und des Fragment Shaders.
 */
void GPUCodeManager::init_shaders(EGL_STATE_T* state) {

	bool res;
	res = loadAndCompileShader("./res/VertexShader.glsl", state->vertexShader, GL_VERTEX_SHADER);
	res = loadAndCompileShader("./res/FragmentShader.glsl", state->fragmentShader, GL_FRAGMENT_SHADER);
}

void GPUCodeManager::createShaderAndProgram(EGL_STATE_T* state) {
	init_shaders(state);
	state->program = glCreateProgram();
        GLES20ErrorHandler::checkError(className, "createShaderAndProgram", "glCreateProgram");

    glAttachShader(state->program, state->vertexShader);
    GLES20ErrorHandler::checkError(className, "createShaderAndProgram", "glAttachShader");
    glAttachShader(state->program, state->fragmentShader);
    GLES20ErrorHandler::checkError(className, "createShaderAndProgram", "glAttachShader");
    glLinkProgram(state->program);
    GLES20ErrorHandler::checkError(className, "createShaderAndProgram", "glLinkProgram");
    // Erfolg des linking muss jetzt abgefragt werden:
    GLint linkStatus;
    glGetProgramiv(state->program, GL_LINK_STATUS, &linkStatus);
    if (linkStatus == GL_FALSE) {
		GLint infoLength;
		glGetProgramiv(state->program, GL_INFO_LOG_LENGTH, &infoLength);
		GLchar buffer[infoLength];
		GLsizei realLength;
		glGetProgramInfoLog(state->program, infoLength,
		                    &realLength,
		                    buffer);
		std::cout << "error message from linker: "
		          << std::endl
		          << buffer
		          << std::endl;
	}

    glUseProgram(state->program); 
    attrLocVertex = glGetAttribLocation(state->program, "vertex");
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glGetAttribLocation");
    uniformLocCenter = glGetUniformLocation(state->program, "center");
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glGetUniformLocation");
    uniformLocScale = glGetUniformLocation(state->program, "scale");
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glGetUniformLocation");
    uniformLocAlpha = glGetUniformLocation(state->program, "alpha");
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glGetUniformLocation");
    state -> hasShader = EGL_TRUE;
}

void GPUCodeManager::drawScene(EGL_STATE_T* state) {
	GLfloat vertices [] = { -1.0,-1.0,
                             1.0,-1.0,
                             1.0,1.0,
                            -1.0,1.0, }; 
    GLfloat min, max;
    max = (GLfloat)(model.getZoomValue()/10.0);
    min = - max;
    glUseProgram(state->program);

    glViewport(0, 0, (GLsizei)state->image_width, (GLsizei)state->image_width);
    glClearDepthf(1.0f);
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    GLES20ErrorHandler::checkError("OpenGLSceneView", "redraw_scene", "glClear");
    
    glUniform2f(uniformLocCenter, state->image_width/2, state->image_width/2);
    GLfloat scale = (max - min)/state->image_width;
    //std::cout << "scale = " << scale << std::endl;
    glUniform2f(uniformLocScale, scale, scale);
    glUniform1f(uniformLocAlpha, (GLfloat) (GLfloat) model.getAngle());
  
    // hier kommt die Bilderzeugung
    glEnableVertexAttribArray(attrLocVertex);  
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glEnableVertexAttribArray");
    glVertexAttribPointer(attrLocVertex, 2, GL_FLOAT, GL_FALSE, 0, &vertices[0]);
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glVertexAttribPointer");

    glDrawArrays(GL_TRIANGLE_FAN, 0, 4);
    GLES20ErrorHandler::checkError(className, "redraw_scene", "glDrawArrays");
    glDisableVertexAttribArray(attrLocVertex);
}

