#include "OpenGLSceneView.h"
#include "EGLInterface.h"
#include <cairomm/context.h>
#include <gdkmm/general.h>   //  set_source_pixbuf()

OpenGLSceneView::OpenGLSceneView(ApplicationModel &m) :
    gpu(m)
{
    state = &_state;
       // Clear application state
    memset( state, 0, sizeof( *state ) );
    imageBuffer = (char) 0;
    state -> hasShader = EGL_FALSE;
    EGLInterface::initEGL();
    EGLInterface::createEGLContext(state);
    // ideal wäre es, die shader hier bereitzustellen
}

//  Alle EGL-Resourcen freigeben
OpenGLSceneView:: ~OpenGLSceneView() {
    glDetachShader(state->program, state->vertexShader);
	glDetachShader(state->program, state->fragmentShader);
	glDeleteShader(state->vertexShader);
	glDeleteShader(state->fragmentShader);
    glDeleteProgram(state->program);
    EGLInterface::freeEGLContext(state);
    delete [] imageBuffer;
    EGLInterface::shutdownEGL(state);
}

void::OpenGLSceneView::redraw_scene(EGL_STATE_T* state){ 
	gpu.drawScene(state);/*
    glFlush();
    glFinish();
    eglWaitClient(); */
    // copy into Gtk world
    glReadPixels(0, 0,
                 state -> image_width,
                 state -> image_width,
                 GL_RGBA,
                 GL_UNSIGNED_BYTE,
                 imageBuffer);
    
    //  Bild aufrichten
    //  Die Bildaufrichtung geschieht zeilenweise in-place
    
    unsigned char lineBuffer [4*state->image_width] ;
    
    int upperlimit = state->image_width/2;
    int sourceIdx, destIdx, lineLength;
    lineLength = 4 * state -> image_width;
    sourceIdx = (state -> image_width - 1)*state -> image_width*4;
    destIdx = 0;
    for (int i = 0; i < upperlimit; i++) {
        memcpy(&lineBuffer[0], &imageBuffer[destIdx], lineLength);
        memmove(&imageBuffer[destIdx], &imageBuffer[sourceIdx], lineLength);
        memcpy(&imageBuffer[sourceIdx], &lineBuffer[0], lineLength);
        sourceIdx -= lineLength;
        destIdx += lineLength;
    } 
}



//
//  Diese Methode wird aufgerufen, wenn sich die Platzierung
//  oder die Groesse des Fensters aendert.
//  Vorhandene Puffer werden zurueckgegeben, neue Puffer in der
//  erforderlichen Groesse werden hergestellt.
//
bool OpenGLSceneView::on_configure_event(GdkEventConfigure* event) { 
    int useableWidth = (get_width()/16) * 16;
    int useableHeight = (get_height()/16)*16;
    int s = useableWidth < useableHeight ? useableWidth : useableHeight;
    state -> screen_width = get_width();
    state -> screen_height = get_height();
    state -> image_width = s;
    
    if (imageBuffer != (char) 0) { 
		free(imageBuffer);
	}

    imageBuffer = new unsigned char [state -> image_width * state -> image_width * 4];
    EGLInterface::resizeEGLBuffer(state);
    if (state -> hasShader == EGL_FALSE) {
		gpu.createShaderAndProgram(state);
	}

    redraw_scene(state);
    return true;
}

//
//  Hier wird das gespeicherte Bild lediglich in einen Pixbuf
//  kopiert und sodann angezeigt.
//
bool OpenGLSceneView::on_draw(const Cairo::RefPtr<Cairo::Context> &context) {
      
   Glib ::RefPtr<Gdk::Pixbuf> pixbuff;
   pixbuff = Gdk::Pixbuf::create_from_data(imageBuffer,
               Gdk::COLORSPACE_RGB, true, 8,
               state -> image_width,
               state -> image_width,
               4*state -> image_width); 
   int x0, y0;
   x0 = (int)(state -> screen_width - state -> image_width)/2;
   y0 = (int)(state -> screen_height - state -> image_width)/2;
   Gdk::Cairo::set_source_pixbuf(context, pixbuff, x0, y0);
   
   context->paint();
   pixbuff.reset();  //  free pixmap
   return true;
}

void OpenGLSceneView::refresh() {
	redraw_scene(state);
   queue_draw();  
}
